package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.Objects;
import java.util.function.BiFunction;
import java.util.regex.MatchResult;
import java.util.regex.Pattern;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

final class TextReplacementConfigImpl implements TextReplacementConfig {
   private final Pattern matchPattern;
   private final BiFunction<MatchResult, TextComponent.Builder, @Nullable ComponentLike> replacement;
   private final TextReplacementConfig.Condition continuer;

   TextReplacementConfigImpl(final Builder builder) {
      this.matchPattern = builder.matchPattern;
      this.replacement = builder.replacement;
      this.continuer = builder.continuer;
   }

   public @NotNull Pattern matchPattern() {
      return this.matchPattern;
   }

   TextReplacementRenderer.State createState() {
      return new TextReplacementRenderer.State(this.matchPattern, this.replacement, this.continuer);
   }

   public TextReplacementConfig.@NotNull Builder toBuilder() {
      return new Builder(this);
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("matchPattern", this.matchPattern), ExaminableProperty.of("replacement", this.replacement), ExaminableProperty.of("continuer", this.continuer));
   }

   public String toString() {
      return Internals.toString(this);
   }

   static final class Builder implements TextReplacementConfig.Builder {
      @Nullable Pattern matchPattern;
      @Nullable BiFunction<MatchResult, TextComponent.Builder, @Nullable ComponentLike> replacement;
      TextReplacementConfig.Condition continuer = (matchResult, index, replacement) -> {
         return PatternReplacementResult.REPLACE;
      };

      Builder() {
      }

      Builder(final TextReplacementConfigImpl instance) {
         this.matchPattern = instance.matchPattern;
         this.replacement = instance.replacement;
         this.continuer = instance.continuer;
      }

      @NotNull
      public @NotNull Builder match(final @NotNull Pattern pattern) {
         this.matchPattern = (Pattern)Objects.requireNonNull(pattern, "pattern");
         return this;
      }

      @NotNull
      public @NotNull Builder condition(final TextReplacementConfig.@NotNull Condition condition) {
         this.continuer = (TextReplacementConfig.Condition)Objects.requireNonNull(condition, "continuation");
         return this;
      }

      @NotNull
      public @NotNull Builder replacement(final @NotNull BiFunction<MatchResult, TextComponent.Builder, @Nullable ComponentLike> replacement) {
         this.replacement = (BiFunction)Objects.requireNonNull(replacement, "replacement");
         return this;
      }

      public @NotNull TextReplacementConfig build() {
         if (this.matchPattern == null) {
            throw new IllegalStateException("A pattern must be provided to match against");
         } else if (this.replacement == null) {
            throw new IllegalStateException("A replacement action must be provided");
         } else {
            return new TextReplacementConfigImpl(this);
         }
      }
   }
}
